@@ -22,15 +22,15 @@
 
 import argparse
 import logging
-from typing import Any, Dict, List, Optional, Union
+from typing import Any
 
 import attr
-
-from synapse._pydantic_compat import (
+from pydantic import (
     StrictBool,
     StrictInt,
     StrictStr,
 )
+
 from synapse.config._base import (
     Config,
     ConfigError,
@@ -79,7 +79,7 @@ MAIN_PROCESS_INSTANCE_MAP_NAME = "main"
 logger = logging.getLogger(__name__)
 
 
-def _instance_to_list_converter(obj: Union[str, List[str]]) -> List[str]:
+def _instance_to_list_converter(obj: str | list[str]) -> list[str]:
     """Helper for allowing parsing a string or list of strings to a config
     option expecting a list of strings.
     """
@@ -119,7 +119,7 @@ class InstanceUnixLocationConfig(ParseModel):
         return f"{self.path}"
 
 
-InstanceLocationConfig = Union[InstanceTcpLocationConfig, InstanceUnixLocationConfig]
+InstanceLocationConfig = InstanceTcpLocationConfig | InstanceUnixLocationConfig
 
 
 @attr.s
@@ -142,39 +142,39 @@ class WriterLocations:
         device_lists: The instances that write to the device list stream.
     """
 
-    events: List[str] = attr.ib(
+    events: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    typing: List[str] = attr.ib(
+    typing: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    to_device: List[str] = attr.ib(
+    to_device: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    account_data: List[str] = attr.ib(
+    account_data: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    receipts: List[str] = attr.ib(
+    receipts: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    presence: List[str] = attr.ib(
+    presence: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    push_rules: List[str] = attr.ib(
+    push_rules: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    device_lists: List[str] = attr.ib(
+    device_lists: list[str] = attr.ib(
         default=[MAIN_PROCESS_INSTANCE_NAME],
         converter=_instance_to_list_converter,
     )
-    thread_subscriptions: List[str] = attr.ib(
+    thread_subscriptions: list[str] = attr.ib(
         default=["master"],
         converter=_instance_to_list_converter,
     )
@@ -190,8 +190,8 @@ class OutboundFederationRestrictedTo:
         locations: list of instance locations to connect to proxy via.
     """
 
-    instances: Optional[List[str]]
-    locations: List[InstanceLocationConfig] = attr.Factory(list)
+    instances: list[str] | None
+    locations: list[InstanceLocationConfig] = attr.Factory(list)
 
     def __contains__(self, instance: str) -> bool:
         # It feels a bit dirty to return `True` if `instances` is `None`, but it makes
@@ -246,7 +246,7 @@ class WorkerConfig(Config):
         if worker_replication_secret_path:
             if worker_replication_secret:
                 raise ConfigError(CONFLICTING_WORKER_REPLICATION_SECRET_OPTS_ERROR)
-            self.worker_replication_secret: Optional[str] = read_file(
+            self.worker_replication_secret: str | None = read_file(
                 worker_replication_secret_path, ("worker_replication_secret_path",)
             ).strip()
         else:
@@ -295,7 +295,7 @@ class WorkerConfig(Config):
         # A map from instance name to host/port of their HTTP replication endpoint.
         # Check if the main process is declared. The main process itself doesn't need
         # this data as it would never have to talk to itself.
-        instance_map: Dict[str, Any] = config.get("instance_map", {})
+        instance_map: dict[str, Any] = config.get("instance_map", {})
 
         if self.instance_name is not MAIN_PROCESS_INSTANCE_NAME:
             # TODO: The next 3 condition blocks can be deleted after some time has
@@ -341,8 +341,8 @@ class WorkerConfig(Config):
                     % MAIN_PROCESS_INSTANCE_MAP_NAME
                 )
 
-        # type-ignore: the expression `Union[A, B]` is not a Type[Union[A, B]] currently
-        self.instance_map: Dict[str, InstanceLocationConfig] = (
+        # type-ignore: the expression `A | B` is not a `type[A | B]` currently
+        self.instance_map: dict[str, InstanceLocationConfig] = (
             parse_and_validate_mapping(
                 instance_map,
                 InstanceLocationConfig,  # type: ignore[arg-type]
@@ -481,7 +481,7 @@ class WorkerConfig(Config):
 
     def _should_this_worker_perform_duty(
         self,
-        config: Dict[str, Any],
+        config: dict[str, Any],
         legacy_master_option_name: str,
         legacy_worker_app_name: str,
         new_option_name: str,
@@ -574,11 +574,11 @@ class WorkerConfig(Config):
 
     def _worker_names_performing_this_duty(
         self,
-        config: Dict[str, Any],
+        config: dict[str, Any],
         legacy_option_name: str,
         legacy_app_name: str,
         modern_instance_list_name: str,
-    ) -> List[str]:
+    ) -> list[str]:
         """
         Retrieves the names of the workers handling a given duty, by either legacy
         option or instance list.
