@@ -30,8 +30,6 @@ from typing import (
     Mapping,
     Match,
     MutableMapping,
-    Optional,
-    Union,
 )
 
 import attr
@@ -415,9 +413,9 @@ class SerializeEventConfig:
     event_format: Callable[[JsonDict], JsonDict] = format_event_for_client_v1
     # The entity that requested the event. This is used to determine whether to include
     # the transaction_id in the unsigned section of the event.
-    requester: Optional[Requester] = None
+    requester: Requester | None = None
     # List of event fields to include. If empty, all fields will be returned.
-    only_event_fields: Optional[list[str]] = None
+    only_event_fields: list[str] | None = None
     # Some events can have stripped room state stored in the `unsigned` field.
     # This is required for invite and knock functionality. If this option is
     # False, that state will be removed from the event before it is returned.
@@ -439,7 +437,7 @@ def make_config_for_admin(existing: SerializeEventConfig) -> SerializeEventConfi
 
 
 def serialize_event(
-    e: Union[JsonDict, EventBase],
+    e: JsonDict | EventBase,
     time_now_ms: int,
     *,
     config: SerializeEventConfig = _DEFAULT_SERIALIZE_EVENT_CONFIG,
@@ -480,7 +478,7 @@ def serialize_event(
     # If we have a txn_id saved in the internal_metadata, we should include it in the
     # unsigned section of the event if it was sent by the same session as the one
     # requesting the event.
-    txn_id: Optional[str] = getattr(e.internal_metadata, "txn_id", None)
+    txn_id: str | None = getattr(e.internal_metadata, "txn_id", None)
     if (
         txn_id is not None
         and config.requester is not None
@@ -490,7 +488,7 @@ def serialize_event(
         # this includes old events as well as those created by appservice, guests,
         # or with tokens minted with the admin API. For those events, fallback
         # to using the access token instead.
-        event_device_id: Optional[str] = getattr(e.internal_metadata, "device_id", None)
+        event_device_id: str | None = getattr(e.internal_metadata, "device_id", None)
         if event_device_id is not None:
             if event_device_id == config.requester.device_id:
                 d["unsigned"]["transaction_id"] = txn_id
@@ -504,9 +502,7 @@ def serialize_event(
             #
             # For guests and appservice users, we can't check the access token ID
             # so assume it is the same session.
-            event_token_id: Optional[int] = getattr(
-                e.internal_metadata, "token_id", None
-            )
+            event_token_id: int | None = getattr(e.internal_metadata, "token_id", None)
             if (
                 (
                     event_token_id is not None
@@ -577,11 +573,11 @@ class EventClientSerializer:
 
     async def serialize_event(
         self,
-        event: Union[JsonDict, EventBase],
+        event: JsonDict | EventBase,
         time_now: int,
         *,
         config: SerializeEventConfig = _DEFAULT_SERIALIZE_EVENT_CONFIG,
-        bundle_aggregations: Optional[dict[str, "BundledAggregations"]] = None,
+        bundle_aggregations: dict[str, "BundledAggregations"] | None = None,
     ) -> JsonDict:
         """Serializes a single event.
 
@@ -712,11 +708,11 @@ class EventClientSerializer:
     @trace
     async def serialize_events(
         self,
-        events: Collection[Union[JsonDict, EventBase]],
+        events: Collection[JsonDict | EventBase],
         time_now: int,
         *,
         config: SerializeEventConfig = _DEFAULT_SERIALIZE_EVENT_CONFIG,
-        bundle_aggregations: Optional[dict[str, "BundledAggregations"]] = None,
+        bundle_aggregations: dict[str, "BundledAggregations"] | None = None,
     ) -> list[JsonDict]:
         """Serializes multiple events.
 
@@ -755,13 +751,13 @@ class EventClientSerializer:
         self._add_extra_fields_to_unsigned_client_event_callbacks.append(callback)
 
 
-_PowerLevel = Union[str, int]
-PowerLevelsContent = Mapping[str, Union[_PowerLevel, Mapping[str, _PowerLevel]]]
+_PowerLevel = str | int
+PowerLevelsContent = Mapping[str, _PowerLevel | Mapping[str, _PowerLevel]]
 
 
 def copy_and_fixup_power_levels_contents(
     old_power_levels: PowerLevelsContent,
-) -> dict[str, Union[int, dict[str, int]]]:
+) -> dict[str, int | dict[str, int]]:
     """Copy the content of a power_levels event, unfreezing immutabledicts along the way.
 
     We accept as input power level values which are strings, provided they represent an
@@ -777,7 +773,7 @@ def copy_and_fixup_power_levels_contents(
     if not isinstance(old_power_levels, collections.abc.Mapping):
         raise TypeError("Not a valid power-levels content: %r" % (old_power_levels,))
 
-    power_levels: dict[str, Union[int, dict[str, int]]] = {}
+    power_levels: dict[str, int | dict[str, int]] = {}
 
     for k, v in old_power_levels.items():
         if isinstance(v, collections.abc.Mapping):
@@ -901,7 +897,7 @@ def strip_event(event: EventBase) -> JsonDict:
     }
 
 
-def parse_stripped_state_event(raw_stripped_event: Any) -> Optional[StrippedStateEvent]:
+def parse_stripped_state_event(raw_stripped_event: Any) -> StrippedStateEvent | None:
     """
     Given a raw value from an event's `unsigned` field, attempt to parse it into a
     `StrippedStateEvent`.
