@@ -1,5 +1,4 @@
 import logging
-from typing import Optional
 
 from synapse.types import JsonMapping
 
@@ -9,7 +8,7 @@ logger = logging.getLogger(__name__)
 class AdminClientConfig:
     """Class to track various Synapse-specific admin-only client-impacting config options."""
 
-    def __init__(self, account_data: Optional[JsonMapping]):
+    def __init__(self, account_data: JsonMapping | None):
         # Allow soft-failed events to be returned down `/sync` and other
         # client APIs. `io.element.synapse.soft_failed: true` is added to the
         # `unsigned` portion of the event to inform clients that the event
