@@ -21,7 +21,7 @@
 
 import logging
 import math
-from typing import Collection, Dict, FrozenSet, List, Mapping, Optional, Set, Union
+from typing import Collection, Mapping
 
 import attr
 from sortedcontainers import SortedDict
@@ -45,14 +45,14 @@ class AllEntitiesChangedResult:
     that callers do the correct checks.
     """
 
-    _entities: Optional[List[EntityType]]
+    _entities: list[EntityType] | None
 
     @property
     def hit(self) -> bool:
         return self._entities is not None
 
     @property
-    def entities(self) -> List[EntityType]:
+    def entities(self) -> list[EntityType]:
         assert self._entities is not None
         return self._entities
 
@@ -73,20 +73,32 @@ class StreamChangeCache:
 
     def __init__(
         self,
+        *,
         name: str,
+        server_name: str,
         current_stream_pos: int,
         max_size: int = 10000,
-        prefilled_cache: Optional[Mapping[EntityType, int]] = None,
+        prefilled_cache: Mapping[EntityType, int] | None = None,
     ) -> None:
+        """
+        Args:
+            name
+            server_name: The homeserver name that this cache is associated with
+                (used to label the metric) (`hs.hostname`).
+            current_stream_pos
+            max_size
+            prefilled_cache
+        """
+
         self._original_max_size: int = max_size
         self._max_size = math.floor(max_size)
 
         # map from stream id to the set of entities which changed at that stream id.
-        self._cache: SortedDict[int, Set[EntityType]] = SortedDict()
+        self._cache: SortedDict[int, set[EntityType]] = SortedDict()
         # map from entity to the stream ID of the latest change for that entity.
         #
         # Must be kept in sync with _cache.
-        self._entity_to_key: Dict[EntityType, int] = {}
+        self._entity_to_key: dict[EntityType, int] = {}
 
         # the earliest stream_pos for which we can reliably answer
         # get_all_entities_changed. In other words, one less than the earliest
@@ -96,7 +108,11 @@ class StreamChangeCache:
 
         self.name = name
         self.metrics = caches.register_cache(
-            "cache", self.name, self._cache, resize_callback=self.set_cache_factor
+            cache_type="cache",
+            cache_name=self.name,
+            server_name=server_name,
+            cache=self._cache,
+            resize_callback=self.set_cache_factor,
         )
 
         if prefilled_cache:
@@ -142,9 +158,9 @@ class StreamChangeCache:
         """
         assert isinstance(stream_pos, int)
 
-        # _cache is not valid at or before the earliest known stream position, so
+        # _cache is not valid before the earliest known stream position, so
         # return that the entity has changed.
-        if stream_pos <= self._earliest_known_stream_pos:
+        if stream_pos < self._earliest_known_stream_pos:
             self.metrics.inc_misses()
             return True
 
@@ -166,7 +182,7 @@ class StreamChangeCache:
 
     def get_entities_changed(
         self, entities: Collection[EntityType], stream_pos: int, _perf_factor: int = 1
-    ) -> Union[Set[EntityType], FrozenSet[EntityType]]:
+    ) -> set[EntityType] | frozenset[EntityType]:
         """
         Returns the subset of the given entities that have had changes after the given position.
 
@@ -186,7 +202,7 @@ class StreamChangeCache:
             This will be all entities if the given stream position is at or earlier
             than the earliest known stream position.
         """
-        if not self._cache or stream_pos <= self._earliest_known_stream_pos:
+        if not self._cache or stream_pos < self._earliest_known_stream_pos:
             self.metrics.inc_misses()
             return set(entities)
 
@@ -238,9 +254,9 @@ class StreamChangeCache:
         """
         assert isinstance(stream_pos, int)
 
-        # _cache is not valid at or before the earliest known stream position, so
+        # _cache is not valid before the earliest known stream position, so
         # return that an entity has changed.
-        if stream_pos <= self._earliest_known_stream_pos:
+        if stream_pos < self._earliest_known_stream_pos:
             self.metrics.inc_misses()
             return True
 
@@ -270,12 +286,12 @@ class StreamChangeCache:
         """
         assert isinstance(stream_pos, int)
 
-        # _cache is not valid at or before the earliest known stream position, so
+        # _cache is not valid before the earliest known stream position, so
         # return None to mark that it is unknown if an entity has changed.
-        if stream_pos <= self._earliest_known_stream_pos:
+        if stream_pos < self._earliest_known_stream_pos:
             return AllEntitiesChangedResult(None)
 
-        changed_entities: List[EntityType] = []
+        changed_entities: list[EntityType] = []
 
         for k in self._cache.islice(start=self._cache.bisect_right(stream_pos)):
             changed_entities.extend(self._cache[k])
@@ -314,6 +330,15 @@ class StreamChangeCache:
         self._entity_to_key[entity] = stream_pos
         self._evict()
 
+    def all_entities_changed(self, stream_pos: int) -> None:
+        """
+        Mark all entities as changed. This is useful when the cache is invalidated and
+        there may be some potential change for all of the entities.
+        """
+        self._cache.clear()
+        self._entity_to_key.clear()
+        self._earliest_known_stream_pos = stream_pos
+
     def _evict(self) -> None:
         """
         Ensure the cache has not exceeded the maximum size.
@@ -327,7 +352,7 @@ class StreamChangeCache:
             for entity in r:
                 self._entity_to_key.pop(entity, None)
 
-    def get_max_pos_of_last_change(self, entity: EntityType) -> Optional[int]:
+    def get_max_pos_of_last_change(self, entity: EntityType) -> int | None:
         """Returns an upper bound of the stream id of the last change to an
         entity.
 
